/*************************************************************************
 *
 *	AUTHOR:	Wilson Tham
 *
 *	TITLE:	button.h
 *
 *	DATE:	April 20, 1993
 *
 *	PURPOSE: Declaration of button class and constants.
 *
 *************************************************************************/

#include <stdlib.h>
#include "typedefs.h"
#include "scrnobj.h"
#include "viewport.h"
#include "display.h"
#include "messages.h"

enum	SHADOW_STYLE {TOP_LEFT, BOTTOM_RIGHT, TOP_RIGHT, BOTTOM_LEFT};
typedef bool BOOLEAN;

				/*************************
				 *                       *
				 * Constant Declarations *
				 *                       *
				 *************************/

const int BORDERX	= 3;  // Distance of button text from button bounds
const int BORDERY	= 2;   // " "
const int SHADOWS	= 2;
const int FRAME	= 1;
const int FONT		= SMALL_FONT;
const int FONT_SIZE	= 5;

class PushButton:public ScrnObj{
	void	*surface;
	int	string_height, string_width, X2, Y2, xdim, ydim;
	char	string[80];
	int	Color, IdNum;
	BOOLEAN Activated, Drawn;

public:
	PushButton			(char*, int, int);
	PushButton			(char*, int, int, int, int);
	void Init				(Display *, int, int);
	~PushButton			()
		{if (Drawn) Erase(); delete surface;}
	void	Draw				();
	void Erase			();
	BOOLEAN ButtonBounds	(int, int);
	BOOLEAN ButtonStatus	();
	int ButtonId			();
	Message Dispatch		(Message &);
	int GetXDim			();
	int GetYDim			();
private:
	void set_constants	(char *, int, int);
	void image_size	();
	void	image_size	(int, int);
	void draw_shadows	(SHADOW_STYLE, int);
	void draw_frame	();
	void name_button	();
	void depress_button	();
	void redraw		();
	void animate_button	();
};

inline PushButton::PushButton(char *name, int color, int Id)
	{set_constants(name, color, Id); image_size();
	 xdim=X2-X; ydim=Y2-Y;}

inline PushButton::PushButton(char *name, int color, int Id, int x2, int y2)
	{set_constants(name, color, Id); image_size(x2, y2);
	 xdim=X2-X; ydim=Y2-Y;}

inline void PushButton::image_size(int x2, int y2)
	{X2 = X+x2; Y2 = Y+y2;}

inline void PushButton::image_size()
	{X2 = X+string_width+(2*BORDERX)+(2*SHADOWS)+(2*FRAME);
	Y2 = Y+string_height+(2*BORDERY)+(2*SHADOWS)+(2*FRAME);}

inline void PushButton::animate_button()
	{if (Activated) depress_button(); else redraw();}

inline BOOLEAN PushButton::ButtonStatus()
	{return Activated;}

inline int PushButton::ButtonId()
	{return IdNum;}

inline int PushButton::GetXDim()
	{return xdim;}

inline int PushButton::GetYDim()
	{return ydim;}
