/*************************************************************************
 *
 *	AUTHOR:	Wilson Tham
 *
 *   TITLE:	checkbox.c
 *
 *	DATE:	April 25, 1993
 *
 *	PURPOSE: 	Checkbox button definitions.
 *
 *************************************************************************/

#include <graphics.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include"checkbox.h"

void CheckBox::set_constants(char *name, int color, int fcolor, int Id, int type)
{
	/*****************************************************************

		Flag states of CheckBoxes...

			Activated - User input detected...
			BStatus	- Whether the option is selected or not...
			Able		- Allow/disallow interaction with user...

	 *****************************************************************/

	Activated		= False;
	BStatus		= Off;	// For now, may be a user-defined field later.
	Able			= Yes;

	Color		= color;
	COLORFILL		= fcolor;
	strcpy(string, name);

	IdNum = Id;
	Drawn = No;
	Type = type;
	X=0; Y=0;

	Viewport surf;
	surf.SetTextStyle(CFONT, HORIZ_DIR, CFONT_SIZE);

	string_height	= surf.TextHeight(string);
	// Adjust for differing fonts...
	string_height	= (int)(string_height+(string_height*.33));
	string_width	= surf.TextWidth(string);
}

void CheckBox::Init(Display *d, int x, int y)
{
	X = x; X2+=X;
	Y = y; Y2+=Y;

	Disp = d;
	BkColor=Disp->GetBkColor();
}

void CheckBox::Draw()
{
	int x=X+CBORDERX;
	int y=(int)Y+abs((string_height-BOXSIZE)/2);

	Disp->FastDraw();
	Disp->SetFillStyle(SOLID_FILL, BOXCOLOR);
	Disp->SetColor(BLACK);

	switch(Type) {
		case Square:
			Disp->Bar(x, y, x+BOXSIZE, y+BOXSIZE);
			Disp->Rectangle(x, y, x+BOXSIZE, y+BOXSIZE);
			break;
		case Circle:
			int radius = (int)(BOXSIZE/2);
			Disp->FillEllipse(x+radius, y+radius, radius, radius);
			break;
	}

	name_button(Color);
	Disp->Done();
	Drawn=Yes;
}

void CheckBox::Erase()
{
	Disp->FastDraw();

	if(Drawn == Yes) {
		Drawn = No;
		Disp->SetFillStyle(SOLID_FILL, Disp->GetBkColor());
		Disp->Bar(X, Y, X2, Y2);
	}
	Disp->Done();
}

void CheckBox::empty_box(int color)
{
	Disp->FastDraw();
	int x=X+CBORDERX;
	int y=(int)Y+abs((string_height-BOXSIZE)/2);

	Disp->SetFillStyle(SOLID_FILL, color);
	switch(Type) {
		case Square:
			Disp->Bar(x+1, y+1, x+BOXSIZE-1, y+BOXSIZE-1);
			break;
		case Circle:
			int radius = (int)(BOXSIZE/2);
			Disp->SetColor(BLACK);
			Disp->FillEllipse(x+radius, y+radius, radius, radius);
			break;
	}
	Disp->Done();

	BStatus = Off;
}

void CheckBox::fill_box()
{
	Disp->FastDraw();
	int x=X+CBORDERX;
	int y=(int)Y+abs((string_height-BOXSIZE)/2);

	Disp->SetFillStyle(SOLID_FILL, COLORFILL);
	Disp->SetColor(COLORFILL);
	switch(Type) {
		case Square:
			draw_check();
			break;
		case Circle:
			int radius = (int)(BOXSIZE/2);
			Disp->FillEllipse(x+radius, y+radius, radius-FILLSIZE-2, radius-FILLSIZE-2);
			break;
	}
	Disp->Done();

	BStatus = On;
}

void CheckBox::draw_check()
{
	int thickness=3;

	int x=X+CBORDERX+FILLSIZE, x2=X+CBORDERX+BOXSIZE-FILLSIZE;
	int y=(int)Y+abs((string_height-BOXSIZE)/2)+FILLSIZE;
	int y2=(int)Y+abs((string_height-BOXSIZE)/2)+BOXSIZE-FILLSIZE;

	int midx=x+(int)(.3333*(x2-x));
	int midy=y+(int)(.6666*(y2-y));

	Disp->SetLineStyle(SOLID_LINE, 0, 1);
	for(int count=0; count<thickness; count++) {
		Disp->Line(x, midy, midx, y2-thickness+count);
		Disp->Line(midx, y2-thickness+count, x2, y);
	}
}

void CheckBox::animate_frame()
{
	int x=X+CBORDERX;
	int y=(int)Y+((string_height-BOXSIZE)/2);
	int radius;

	Disp->FastDraw();

	if (Activated)
	  Disp->SetColor(BLACK);
	else
	  Disp->SetColor(BkColor);

	Disp->SetLineStyle(SOLID_LINE, 0, CFRAME);
	switch(Type) {
		case Square:
			Disp->Rectangle(x-CFRAME+1, y-CFRAME+1, x+BOXSIZE+CFRAME-1, y+BOXSIZE+CFRAME-1);
			break;
		case Circle:
			radius = (int)(BOXSIZE/2);
			Disp->Circle(x+radius, y+radius, radius+CFRAME-1);
			break;
	}

	Disp->Done();
}

BOOLEAN CheckBox::ButtonBounds(int Mx, int My)
{
	// Mouse will call and supply the coordinates of its position
	//	It's up to the button itself to determine it's bounds.

	if ((Mx<=X2) && (Mx>=X) && (My<=Y2) && (My>=Y))
		return True;
	else
		return False;
}

void CheckBox::name_button(int color)
{
	int x, y, x2, y2;

	x	= X+CBORDERX+BOXSIZE+BOXBORDER;
	y	= Y;
	x2	= X2;
	y2	= Y2;

	Viewport surf (Disp, x, y, x2, y2);

	surf.SetColor(color);
	surf.SetTextStyle (CFONT, HORIZ_DIR, CFONT_SIZE);
	surf.OutTextXY(0, 0, string);
}

Message CheckBox::Dispatch(Message &msg)
{
	switch(msg.origin) {
		case MouseMsg:
			return mouse_msg(msg);
		case KeyMsg:
			return key_msg(msg);
		case GodMsg:
			return command_msg(msg);
	}
	return msg;
}

Message CheckBox::mouse_msg(Message &msg)
{
	if (ButtonBounds(msg.d1, msg.d2) && Able) {
		switch(msg.code) {
			case LMouseDown:
				Activated = True;
				animate_frame();
				return Acknowledged;
			case LMouseStillDown:
				if(!Activated) {
					Activated = True;
					animate_frame();
				}
				return NullMsg;
			case LMouseUp:
				Activated = False;
				BStatus = BStatus? Off : On;
				animate_frame();
				animate_box();
				return Message(ButtonMsg, Status, IdNum, BStatus);
			default:
				return NullMsg;
		}
	}
	if(msg.code==LMouseStillDown && Activated && Able) {
		Activated = False;
		animate_frame();
		animate_box();
	}
	return NullMsg;
}

Message CheckBox::key_msg(Message &)
{
/*	if (KeyBounds(msg.d1, msg.d2) && Able)
			return NullMsg; */

return NullMsg;
}

Message CheckBox::command_msg(Message &msg)
{
	if(msg.d1 != IdNum)
		return NullMsg;

	switch (msg.code) {
		case Disable:
			BStatus=Off;
			animate_box();
			disable_box();
			return Acknowledged;
		case Enable:
			enable_box();
			return Acknowledged;
		case RequestStatus:
			return Message(ButtonMsg, Status, IdNum, (int)status());
	}
	return NullMsg;
}

