/**********************************************************************
*       FILE:     MAP.h
*       AUTHOR:   E. Otis Owens
*       DATE:     17 April 1993
*       MODIFIED: 30 April 1993 (5:00 pm)
*       PURPOSE:
*         Functions to store and relay information about the "Scotland
*         Yard" network
***********************************************************************/

#ifndef _MAP_H
#define _MAP_H

#include <fstream.h>
#include "mapnode.h"

/**********************************************************************
*       Class Definition
***********************************************************************/

  const int  FIRST= 1;  // node-count starts from this value
  const char FIELD= ';';
  const char SUB1= '(';
  const char SUB2= ')';
  const char EOLN= '\n';

class Map {

 private:

  int       numNodes,  // number of nodes on the map
            maxX,
            maxY;

  MapNode   *nodes;    // array to store adjecency list
  ifstream  mapFile;   // input file

  void initialize(int *array, int size);
  void get_field(int, mode);

 public:
  Map(void) { numNodes= 0; nodes= NULL; maxX= 0; maxY= 0; }
  ~Map()    { delete [] nodes; }

  bool load_map(char*);
  void show_map(void);  // a debugging function

  int max_x(void) { return maxX; }
  int max_y(void) { return maxY; }

  int x(int node) { return nodes[node].x; }
  int y(int node) { return nodes[node].y; }

  int first_node(void) { return FIRST; }
  int num_nodes(void)  { return numNodes; }

  // connected_to(node, iteration, <iteration>)
  int   connected_to(int, int, int= -1);
  int   num_connections(int, int= -1);
  char* get_name(int node) { return nodes[node].find_name(node); }

  int next_short(int, int, bool, bool, bool);

  bool taxi(int node)   { return nodes[node].is_link(Taxi); }
  bool bus(int node)    { return nodes[node].is_link(Bus); }
  bool subway(int node) { return nodes[node].is_link(Subway); }

  bool taxi(int fromNode, int toNode)
    { return nodes[fromNode].is_link(toNode, Taxi); }

  bool bus(int fromNode, int toNode)
    { return nodes[fromNode].is_link(toNode, Bus); }

  bool subway(int fromNode, int toNode)
    { return nodes[fromNode].is_link(toNode, Subway); }

  vect vehicle(int fromNode, int toNode)
    { return nodes[fromNode].find_vector(toNode); }
};


extern Map theMap;


#endif





