/*************************************************************************
* 	FILE:	moves.h
*	AUTHOR: CALeague
*	DATE:	29 April 1993
*	PURPOSE:
*	  Declares a struct which represents a move:  a destination
*	  node and a mode of transportation.
*	  The struct can also be used to represent the connections between
*	  any two nodes, because the transportation is a bit vector.
**************************************************************************/

#ifndef _MOVE_H
#define _MOVE_H

#include "typedefs.h"


/**********************************************************************
*	Token Types
***********************************************************************/

typedef enum mode TokenType;
const byte NTokenTypes = 3;


/**********************************************************************
*	STRUCT:  Move
***********************************************************************/

struct Move {
  int  Source;		// Source node;
  int  Node;		// Destination node
  byte Trans;		// Transportation type (bit vector)
  Move* Next;		// (for linking)
  Move (void) { Source = Node = Trans = 0;  Next = 0; }
  Move (int s, int n, int t)
    { Source = s; Node = n; Trans = t; Next = 0; }
  int operator == (Move& m)
    { return ((Source==m.Source)&&(Node==m.Node)&&(Trans==m.Trans)); }
  int operator != (Move& m)
    { return ((Source!=m.Source)||(Node!=m.Node)||(Trans!=m.Trans)); }
  ~Move (void) { delete Next; }
};

const Move NullMove;

/**********************************************************************
*	Bit Masks
*
*  	To set bit:
*	  Trans |= <Mask>
*	Or, to change entire vector:
*	  Trans = <Mask1> | <Mask2> | ...
*	To check whether bit is set:
*	  (bool) Trans & <Mask>
*
*	Where <Mask> is one of the following bit masks:
***********************************************************************/

const byte TaxiMask   = 0x01;
const byte BusMask    = 0x02;
const byte SubwayMask = 0x04;


/**********************************************************************
*	CLASS:  MoveList
***********************************************************************/

class MoveList
{
  Move *First;
  Move *Last;
  Move *Current;
  int Len;
  void Insert (Move *a);
public:
  MoveList () { First = Last = Current = 0; Len = 0;}
  ~MoveList () { Nuke (); }
  void Append (Move *a);
  Move* GetCurrent (void) { return Current; }
  Move* GetLast (void) { return Last; }
  bool Reset (void);
  bool Next (void);
  int  Length (void) { return Len; }
  void Nuke (void);
};


#endif
