/*************************************************************************
 *
 *	AUTHOR:	Wilson Tham
 *
 *   TITLE:	nodes.c
 *
 *	DATE:	April 28, 1993
 *
 *	PURPOSE: 	Nodes definitions.
 *
 *************************************************************************/

#include <graphics.h>
#include <stdio.h>
#include <math.h>

#include"nodes.h"
#include"map.h"

void Nodes::set_constants(int Id, int colorfill, BOOLEAN state)
{
	/*****************************************************************

		Flag states of Nodes...

			Activated - User input detected...
			BStatus	- Whether the option is selected or not...
			Able		- Allow/disallow interaction with user...

	 *****************************************************************/

	Activated		= False;
	BStatus		= Off;	// For now, may be a user-defined field later.
	Able			= state;

	NCOLORFILL=colorfill;
	IdNum = Id;
	Drawn = No;
	X=0; Y=0;
}

void Nodes::Init(Display *d, int x, int y)
{
	NBOXSIZE=CCIRCLE+2*STOPRAD;

	X = x-NBOXSIZE/2; X2=X+NBOXSIZE;
	Y = y-NBOXSIZE/2; Y2=Y+NBOXSIZE;

	Disp = d;
	BkColor=Disp->GetBkColor();
}

void Nodes::Draw()
{
	Disp->FastDraw();

	Disp->SetColor(BLACK);

	if(theMap.taxi(IdNum))	// Taxi Stop
		Disp->SetFillStyle(SOLID_FILL, YELLOW);
	else
		Disp->SetFillStyle(SOLID_FILL, GREEN);
	Disp->FillEllipse(X+NBOXSIZE/2, Y+NBOXSIZE/2-CCIRCLE/2+STOPRAD, STOPRAD, STOPRAD);

	if(theMap.bus(IdNum))	// Bus Stop
		Disp->SetFillStyle(SOLID_FILL, GREEN);
	else
		Disp->SetFillStyle(SOLID_FILL, YELLOW);
	Disp->FillEllipse(X+NBOXSIZE/2+CCIRCLE/2, Y+NBOXSIZE/2+2*STOPRAD-2, STOPRAD, STOPRAD);

	if(theMap.subway(IdNum))	// Subway Stop
		Disp->SetFillStyle(SOLID_FILL, RED);
	else if (theMap.taxi (IdNum))
		Disp->SetFillStyle(SOLID_FILL, YELLOW);
	else
		Disp->SetFillStyle(SOLID_FILL, GREEN);
	Disp->FillEllipse(X+NBOXSIZE/2-CCIRCLE/2, Y+NBOXSIZE/2+2*STOPRAD-2, STOPRAD, STOPRAD);

	if(Able)
		Disp->SetFillStyle(SOLID_FILL, NBOXCOLOR);
	else
		Disp->SetFillStyle(SOLID_FILL, NOFFCOLOR);

	int radius = (int)(CCIRCLE/2);
	Disp->SetColor(BLACK);
	Disp->FillEllipse(X+NBOXSIZE/2, Y+NBOXSIZE/2+STOPRAD, radius, radius);

	if(BStatus)
		fill_box();

	Disp->Done();
	Drawn=Yes;
}

void Nodes::empty_box(int color)
{
	Disp->FastDraw();

	Disp->SetFillStyle(SOLID_FILL, color);
	int radius = (int)(CCIRCLE/2);
	Disp->SetColor(BLACK);
	Disp->FillEllipse(X+NBOXSIZE/2, Y+NBOXSIZE/2+STOPRAD, radius, radius);

	Disp->Done();

	BStatus = Off;
}

void Nodes::fill_box()
{
	Disp->FastDraw();

	Disp->SetFillStyle(SOLID_FILL, NCOLORFILL);
	Disp->SetColor(NCOLORFILL);

	int radius = (int)(CCIRCLE/2);
	Disp->SetColor(BLACK);
	Disp->FillEllipse(X+NBOXSIZE/2, Y+NBOXSIZE/2+STOPRAD, radius-NFILLSIZE, radius-NFILLSIZE);

	Disp->Done();

	BStatus = On;
}

void Nodes::animate_frame()
{
	Disp->FastDraw();

	if (Activated)
	  Disp->SetColor(BLACK);
	else
	  Disp->SetColor(NBOXCOLOR);

	Disp->SetLineStyle(SOLID_LINE, 0, NFRAME);

	int radius = (int)(CCIRCLE/2);
	Disp->Circle(X+NBOXSIZE/2, Y+NBOXSIZE/2+STOPRAD, radius-1);

	Disp->Done();
}

Message Nodes::Dispatch(Message &msg)
{
	switch(msg.origin) {
		case MouseMsg:
			return mouse_msg(msg);
		case KeyMsg:
			return key_msg(msg);
		case GodMsg:
			return command_msg(msg);
	}
	return msg;
}

Message Nodes::mouse_msg(Message &msg)
{
	if (ButtonBounds(msg.d1, msg.d2) && Able) {
		switch(msg.code) {
			case LMouseDown:
				Activated = True;
				animate_frame();
				return Acknowledged;
			case LMouseStillDown:
				if(!Activated) {
					Activated = True;
					animate_frame();
				}
				return NullMsg;
			case LMouseUp:
				Activated = False;
				BStatus = BStatus? Off : On;
				animate_frame();
				animate_box();
				return Message(ButtonMsg, Status, IdNum, BStatus);
			default:
				return NullMsg;
		}
	}
	if(msg.code==LMouseStillDown && Activated && Able) {
		Activated = False;
		animate_frame();
		animate_box();
	}
	return NullMsg;
}

Message Nodes::command_msg(Message &msg)
{
	if(msg.d1 != IdNum)
		return NullMsg;

	switch (msg.code) {
		case Disable:
			animate_box();
			disable_box();
			return Acknowledged;
		case Enable:
			enable_box();
			return Acknowledged;
		case RequestStatus:
			return Message(ButtonMsg, Status, IdNum, (int)status());
	}
	return NullMsg;
}

Message Nodes::key_msg(Message &)
{
/*	if (KeyBounds(msg.d1, msg.d2) && Able)
			return NullMsg; */

return NullMsg;
}

