/*************************************************************************
 *
 *	AUTHOR:	Wilson Tham
 *
 *	TITLE:	nodes.h
 *
 *	DATE:	April 28, 1993
 *
 *	PURPOSE:  Declaration of nodes class and constants.
 *
 *************************************************************************/

#ifndef _NODES_H
#define _NODES_H

#include <stdlib.h>
#include "typedefs.h"
#include "scrnobj.h"
#include "viewport.h"
#include "display.h"
#include "messages.h"
#include "checkbox.h"


				/*************************
				 *                       *
				 * Constant Declarations *
				 *                       *
				 *************************/

const int NFRAME	= 1;  // Additional highlighting of node when Active
const int CCIRCLE	= 8;
const int NFILLSIZE	= 1;  // Border of fill graphics from edge
const int STOPRAD	= 3;
const int NBOXCOLOR	= LIGHTGRAY;
const int NOFFCOLOR	= DARKGRAY;

class Nodes:public CheckBox {
	int	NCOLORFILL, NBOXSIZE;
public:
	Nodes				(int, BOOLEAN);
	Nodes				(int, int, BOOLEAN);
	void Init			(Display *, int, int);
	~Nodes				()
		{if (Drawn) Erase();}
	Message Dispatch		(Message &);
	void	Draw			();
	void NewFillColor		(int);
	void EnableNode			(int);
	void DisableNode		();
	void ChangeStatus		(BOOLEAN);
	void ChangeStatus		(BOOLEAN, int);
private:
	void set_constants	(int, int, BOOLEAN);
	void	image_size	();
	void	image_size	(int, int);
	void prep_screen	(int);
	void fill_box		();
	void empty_box		(int);
	void animate_box	();
	void animate_frame	();
	Message key_msg	(Message &);
	Message command_msg	(Message &);
	Message mouse_msg	(Message &);
	void disable_box	();
	void enable_box	();
};

inline Nodes::Nodes(int Id, int colorfill, BOOLEAN state)
	{set_constants(Id, colorfill, state); image_size();}

inline Nodes::Nodes(int Id, BOOLEAN state)
	{set_constants(Id, BLUE, state); image_size();}

inline void Nodes::NewFillColor(int newcolor)
	{NCOLORFILL=newcolor;}

inline void Nodes::DisableNode()
	{disable_box();}

inline void Nodes::EnableNode(int color)
	{if(!BStatus) NCOLORFILL=color; enable_box();}

inline void Nodes::ChangeStatus(BOOLEAN bswitch, int color)
	{BStatus=bswitch; NCOLORFILL=color; animate_box();}

inline void Nodes::ChangeStatus(BOOLEAN bswitch)
	{BStatus=bswitch; animate_box();}

inline void Nodes::image_size()
	{X2 = X+NBOXSIZE; Y2 = Y+NBOXSIZE;}

inline void Nodes::animate_box()
	{if (BStatus) fill_box(); else if(Able) empty_box(NBOXCOLOR);
	 else if(!Able) empty_box(NOFFCOLOR);}

inline void Nodes::disable_box()
	{Able=Off; BOOLEAN temp=BStatus; empty_box(NOFFCOLOR);
	 BStatus=temp; if(BStatus) fill_box();}

inline void Nodes::enable_box()
	{Able=On; BOOLEAN temp=BStatus; empty_box(NBOXCOLOR);
	 BStatus=temp; if(BStatus) fill_box();}

#endif