/**********************************************************************
*	FILE:	random.h
*	AUTHOR:	Christopher League
*	DATE:   15 March 1993
*	PURPOSE:
*	  Declares a class/object for generating random numbers.
***********************************************************************/
 
#ifndef _RANDOM_H
#define _RANDOM_H
 
#include <stdlib.h>
#include <time.h>
 
/**********************************************************************
*	CLASS:  Randomizer
*
*	PURPOSE:
*	  The Randomizer class simply provides a nice interface for the
*	  rand and srand functions.  Also, since the only instance of the
*	  class is global, the random number generator automatically gets
*	  seeded at the beginning of the program.
*
*	FUNCTIONS:
*	  Value		Returns a random value between 0 and the integer
*			  limit-1, inclusive.
*	  operator*	A different interface for Value:
*			  int x = Random * 12;
***********************************************************************/
 
class Randomizer
{
  private:
    time_t seed;
  public:
    Randomizer (void);
    Value (int limit)  { return (rand () % limit); }
    friend int operator* (Randomizer R, int limit)
      { return R.Value (limit); }
};
 
 
extern Randomizer Random;		// Global instance
 
 
#endif
