/*************************************************************************
* 	FILE:	textwin.c
*	AUTHOR: CALeague
*	DATE:	26 April 1993
*	PURPOSE:
*	  A Text Window is a pop-up window which is automatically sized
*	  to contain given text.
**************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "textwin.h"


TextWin::TextWin (int minxdim, int addy, int bc, int bkc,
		char **txt, int txtc)
{
  MinXDim = minxdim;  AddY = addy;
  BorderColor = bc;
  BkColor = bkc;
  TextColor = txtc;
  Text = txt;
  CalcDim ();
}


TextWin::TextWin (int minxdim, int addy, int bc, int bkc, char* t, int tc,
		  char **txt, int txtc)
{
  MinXDim = minxdim;  AddY = addy;
  BorderColor = bc;
  BkColor = bkc;
  TitleColor = tc;
  TextColor = txtc;
  Title = new char [strlen (t)];
  assert (Title != 0);
  strcpy (Title, t);
  Text = txt;
  CalcDim ();
}


void TextWin::CalcDim (void)
{
  Viewport temp;
  temp.SetTextStyle (TextWinFont, HORIZ_DIR, TextWinFontSize);
  THeight = temp.TextHeight ("My") + 4;
  int nlines;

  XDim = MinXDim;
  for (nlines=0; strlen (Text [nlines]); nlines++)
  {
    XDim = max (XDim, temp.TextWidth (Text [nlines]) + 10);
    YDim += THeight;
  }
  if (Title)
    YDim += TitleWidth + 1;
  YDim += AddY + THeight + (BorderWidth*2) + 2;
  XDim += (BorderWidth*2) + 10;
}


void TextWin::Draw (void)
{
  if (!Drawn)
  {
    assert (Disp != NULL);
    Disp->FastDraw ();
    PrepScreen ();
    DrawBorder ();
    if (TitleD)
      DrawTitle ();
    ClientD->Clear ();
    ClientD->SetColor (TextColor);
    ClientD->SetTextStyle (TextWinFont, HORIZ_DIR, TextWinFontSize);
    ClientD->SetTextJustify (CENTER_TEXT, TOP_TEXT);
    ClientD->MoveTo (ClientD->GetMaxX()/2, THeight/2);
    for (int i=0; strlen (Text [i]); i++)
    {
      ClientD->OutText (Text [i]);
      ClientD->MoveRel (0, THeight);
    }
    ClientD->Draw ();
    Disp->Done ();
    Drawn = True;
  }
}
